part of 'table_widget.dart';

typedef TableLoadFn<M> = Future<List<M>> Function(int page, int pageSize);

sealed class TableLoader<M> extends Equatable {
  const TableLoader();
}

final class TableStaticLoader<M> extends TableLoader<M> {
  final FutureOr<List<M>> Function() items;

  const TableStaticLoader(this.items);

  @override
  List<Object?> get props => [items];
}

final class TablePaginatedLoader<M> extends TableLoader<M> {
  final TableLoadFn<M> paginate;

  const TablePaginatedLoader(this.paginate);

  @override
  List<Object?> get props => [paginate];
}

class TableController<M extends Object> {
  final TableLoader<M> loadFn;

  TableController({
    required this.loadFn,
  });

  late TableDataSource<M> dataSource;

  Future<void> reload() async {
    return dataSource.reset();
  }

  Future<void> reloadCurrentPage() async {
    return dataSource.reset(keepOffset: true);
  }

  List<M> get allItems {
    return dataSource.config.pages.values.expand((element) => element).toList();
  }
}
